@@ -3,6 +3,8 @@
 #  Copyright (c) 2020 Seven Bridges
 #  See https://github.com/rabix/sbpack/blob/b8404a0859ffcbe1edae6d8f934e51847b003320/LICENSE
 """
+CWL document packing functions.
+
 The link resolution is as follows:
 
 We always have two components: the base and the link
@@ -13,9 +15,7 @@ used to fetch data
 From https://github.com/rabix/sbpack/blob/b8404a0859ffcbe1edae6d8f934e51847b003320/sbpack/lib.py
 """
 
-
 import logging
-import os
 import sys
 import urllib.parse
 import urllib.request
@@ -216,7 +216,6 @@ def resolve_steps(
     if cwl.get("class") != "Workflow":
         return cwl
 
-    workflow_id = cwl.get("id", os.path.basename(base_url.path))
     for _, v in enumerate(cwl["steps"]):
         if isinstance(v, dict):
             sys.stderr.write(
