@@ -1,16 +1,30 @@
 # SPDX-License-Identifier: Apache-2.0
 import hashlib
 import logging
+import os
 from collections import namedtuple
-from typing import IO, Any, Dict, List, MutableSequence, Optional, Tuple, Union, cast
+from io import StringIO
+from typing import (
+    IO,
+    Any,
+    Dict,
+    List,
+    MutableMapping,
+    MutableSequence,
+    Optional,
+    Tuple,
+    Union,
+    cast,
+)
+from urllib.parse import urldefrag
 
 from ruamel import yaml
 from schema_salad.exceptions import ValidationException
-from schema_salad.sourceline import SourceLine
-from schema_salad.utils import aslist, json_dumps
+from schema_salad.sourceline import SourceLine, add_lc_filename
+from schema_salad.utils import aslist, json_dumps, yaml_no_ts
 
 import cwl_utils.parser
-import cwl_utils.parser.cwl_v1_2 as cwl
+import cwl_utils.parser.cwl_v1_1 as cwl
 import cwl_utils.parser.utils
 from cwl_utils.errors import WorkflowException
 
@@ -30,9 +44,9 @@ def _compare_records(
     This handles normalizing record names, which will be relative to workflow
     step, so that they can be compared.
     """
-    srcfields = {cwl.shortname(field.name): field.type for field in (src.fields or {})}
+    srcfields = {cwl.shortname(field.name): field.type_ for field in (src.fields or {})}
     sinkfields = {
-        cwl.shortname(field.name): field.type for field in (sink.fields or {})
+        cwl.shortname(field.name): field.type_ for field in (sink.fields or {})
     }
     for key in sinkfields.keys():
         if (
@@ -63,10 +77,10 @@ def _compare_type(type1: Any, type2: Any) -> bool:
         return _compare_type(type1.items, type2.items)
     elif isinstance(type1, cwl.RecordSchema) and isinstance(type2, cwl.RecordSchema):
         fields1 = {
-            cwl.shortname(field.name): field.type for field in (type1.fields or {})
+            cwl.shortname(field.name): field.type_ for field in (type1.fields or {})
         }
         fields2 = {
-            cwl.shortname(field.name): field.type for field in (type2.fields or {})
+            cwl.shortname(field.name): field.type_ for field in (type2.fields or {})
         }
         if fields1.keys() != fields2.keys():
             return False
@@ -82,13 +96,62 @@ def _compare_type(type1: Any, type2: Any) -> bool:
         return bool(type1 == type2)
 
 
-def _is_conditional_step(
-    param_to_step: Dict[str, cwl.WorkflowStep], parm_id: str
-) -> bool:
-    if (source_step := param_to_step.get(parm_id)) is not None:
-        if source_step.when is not None:
-            return True
-    return False
+def _inputfile_load(
+    doc: Union[str, MutableMapping[str, Any], MutableSequence[Any]],
+    baseuri: str,
+    loadingOptions: cwl.LoadingOptions,
+    addl_metadata_fields: Optional[MutableSequence[str]] = None,
+) -> Tuple[Any, cwl.LoadingOptions]:
+    loader = cwl.CWLInputFileLoader
+    if isinstance(doc, str):
+        url = loadingOptions.fetcher.urljoin(baseuri, doc)
+        if url in loadingOptions.idx:
+            return loadingOptions.idx[url]
+        doc_url, frg = urldefrag(url)
+        text = loadingOptions.fetcher.fetch_text(doc_url)
+        textIO = StringIO(text)
+        textIO.name = str(doc_url)
+        yaml = yaml_no_ts()
+        result = yaml.load(textIO)
+        add_lc_filename(result, doc_url)
+        loadingOptions = cwl.LoadingOptions(copyfrom=loadingOptions, fileuri=doc_url)
+        _inputfile_load(
+            result,
+            doc_url,
+            loadingOptions,
+        )
+        return loadingOptions.idx[url]
+
+    if isinstance(doc, MutableMapping):
+        addl_metadata = {}
+        if addl_metadata_fields is not None:
+            for mf in addl_metadata_fields:
+                if mf in doc:
+                    addl_metadata[mf] = doc[mf]
+
+        loadingOptions = cwl.LoadingOptions(
+            copyfrom=loadingOptions,
+            baseuri=baseuri,
+            addl_metadata=addl_metadata,
+        )
+
+        loadingOptions.idx[baseuri] = (
+            loader.load(doc, baseuri, loadingOptions, docRoot=baseuri),
+            loadingOptions,
+        )
+
+        return loadingOptions.idx[baseuri]
+
+    if isinstance(doc, MutableSequence):
+        loadingOptions.idx[baseuri] = (
+            loader.load(doc, baseuri, loadingOptions),
+            loadingOptions,
+        )
+        return loadingOptions.idx[baseuri]
+
+    raise ValidationException(
+        "Expected URI string, MutableMapping or MutableSequence, got %s" % type(doc)
+    )
 
 
 def can_assign_src_to_sink(src: Any, sink: Any, strict: bool = False) -> bool:
@@ -129,15 +192,11 @@ def can_assign_src_to_sink(src: Any, sink: Any, strict: bool = False) -> bool:
 def check_all_types(
     src_dict: Dict[str, Any],
     sinks: MutableSequence[Union[cwl.WorkflowStepInput, cwl.WorkflowOutputParameter]],
-    param_to_step: Dict[str, cwl.WorkflowStep],
     type_dict: Dict[str, Any],
 ) -> Dict[str, List[SrcSink]]:
     """Given a list of sinks, check if their types match with the types of their sources."""
     validation: Dict[str, List[SrcSink]] = {"warning": [], "exception": []}
     for sink in sinks:
-        extra_message = (
-            "pickValue is %s" % sink.pickValue if sink.pickValue is not None else None
-        )
         if isinstance(sink, cwl.WorkflowOutputParameter):
             sourceName = "outputSource"
             sourceField = sink.outputSource
@@ -151,23 +210,9 @@ def check_all_types(
                 linkMerge = sink.linkMerge or (
                     "merge_nested" if len(sourceField) > 1 else None
                 )
-                if sink.pickValue in ["first_non_null", "the_only_non_null"]:
-                    linkMerge = None
                 srcs_of_sink = []
                 for parm_id in sourceField:
                     srcs_of_sink += [src_dict[parm_id]]
-                    if (
-                        _is_conditional_step(param_to_step, parm_id)
-                        and sink.pickValue is not None
-                    ):
-                        validation["warning"].append(
-                            SrcSink(
-                                src_dict[parm_id],
-                                sink,
-                                linkMerge,
-                                message="Source is from conditional step, but pickValue is not used",
-                            )
-                        )
             else:
                 parm_id = cast(str, sourceField)
                 if parm_id not in src_dict:
@@ -176,33 +221,6 @@ def check_all_types(
                     )
                 srcs_of_sink = [src_dict[parm_id]]
                 linkMerge = None
-                if sink.pickValue is not None:
-                    validation["warning"].append(
-                        SrcSink(
-                            src_dict[parm_id],
-                            sink,
-                            linkMerge,
-                            message="pickValue is used but only a single input source is declared",
-                        )
-                    )
-                if _is_conditional_step(param_to_step, parm_id):
-                    src_typ = aslist(type_dict[srcs_of_sink[0].id])
-                    snk_typ = type_dict[cast(str, sink.id)]
-                    if "null" not in src_typ:
-                        src_typ = ["null"] + cast(List[Any], src_typ)
-                    if (
-                        not isinstance(snk_typ, MutableSequence)
-                        or "null" not in snk_typ
-                    ):
-                        validation["warning"].append(
-                            SrcSink(
-                                src_dict[parm_id],
-                                sink,
-                                linkMerge,
-                                message="Source is from conditional step and may produce `null`",
-                            )
-                        )
-                    type_dict[srcs_of_sink[0].id] = src_typ
             for src in srcs_of_sink:
                 check_result = check_types(
                     type_dict[cast(str, src.id)],
@@ -211,13 +229,9 @@ def check_all_types(
                     getattr(sink, "valueFrom", None),
                 )
                 if check_result == "warning":
-                    validation["warning"].append(
-                        SrcSink(src, sink, linkMerge, extra_message)
-                    )
+                    validation["warning"].append(SrcSink(src, sink, linkMerge, None))
                 elif check_result == "exception":
-                    validation["exception"].append(
-                        SrcSink(src, sink, linkMerge, extra_message)
-                    )
+                    validation["exception"].append(SrcSink(src, sink, linkMerge, None))
     return validation
 
 
@@ -242,7 +256,7 @@ def check_types(
         return "exception"
     if linkMerge == "merge_nested":
         return check_types(
-            cwl.ArraySchema(items=srctype, type="array"), sinktype, None, None
+            cwl.ArraySchema(items=srctype, type_="array"), sinktype, None, None
         )
     if linkMerge == "merge_flattened":
         return check_types(merge_flatten_type(srctype), sinktype, None, None)
@@ -253,21 +267,16 @@ def content_limit_respected_read_bytes(f: IO[bytes]) -> bytes:
     """
     Read file content up to 64 kB as a byte array.
 
-    Throw exception for larger files (see https://www.commonwl.org/v1.2/Workflow.html#Changelog).
+    Truncate content for larger files.
     """
-    contents = f.read(CONTENT_LIMIT + 1)
-    if len(contents) > CONTENT_LIMIT:
-        raise WorkflowException(
-            "file is too large, loadContents limited to %d bytes" % CONTENT_LIMIT
-        )
-    return contents
+    return f.read(CONTENT_LIMIT)
 
 
 def content_limit_respected_read(f: IO[bytes]) -> str:
     """
     Read file content up to 64 kB as an utf-8 encoded string.
 
-    Throw exception for larger files (see https://www.commonwl.org/v1.2/Workflow.html#Changelog).
+    Truncate content for larger files.
     """
     return content_limit_respected_read_bytes(f).decode("utf-8")
 
@@ -275,7 +284,7 @@ def content_limit_respected_read(f: IO[bytes]) -> str:
 def convert_stdstreams_to_files(clt: cwl.CommandLineTool) -> None:
     """Convert stdin, stdout and stderr type shortcuts to files."""
     for out in clt.outputs:
-        if out.type == "stdout":
+        if out.type_ == "stdout":
             if out.outputBinding is not None:
                 raise ValidationException(
                     "Not allowed to specify outputBinding when using stdout shortcut."
@@ -286,9 +295,9 @@ def convert_stdstreams_to_files(clt: cwl.CommandLineTool) -> None:
                         json_dumps(clt.save(), sort_keys=True).encode("utf-8")
                     ).hexdigest()
                 )
-            out.type = "File"
+            out.type_ = "File"
             out.outputBinding = cwl.CommandOutputBinding(glob=clt.stdout)
-        elif out.type == "stderr":
+        elif out.type_ == "stderr":
             if out.outputBinding is not None:
                 raise ValidationException(
                     "Not allowed to specify outputBinding when using stderr shortcut."
@@ -299,10 +308,10 @@ def convert_stdstreams_to_files(clt: cwl.CommandLineTool) -> None:
                         json_dumps(clt.save(), sort_keys=True).encode("utf-8")
                     ).hexdigest()
                 )
-            out.type = "File"
+            out.type_ = "File"
             out.outputBinding = cwl.CommandOutputBinding(glob=clt.stderr)
     for inp in clt.inputs:
-        if inp.type == "stdin":
+        if inp.type_ == "stdin":
             if inp.inputBinding is not None:
                 raise ValidationException(
                     "Not allowed to specify unputBinding when using stdin shortcut."
@@ -316,7 +325,66 @@ def convert_stdstreams_to_files(clt: cwl.CommandLineTool) -> None:
                     "$(inputs.%s.path)"
                     % cast(str, inp.id).rpartition("#")[2].split("/")[-1]
                 )
-                inp.type = "File"
+                inp.type_ = "File"
+
+
+def load_inputfile(
+    doc: Any,
+    baseuri: Optional[str] = None,
+    loadingOptions: Optional[cwl.LoadingOptions] = None,
+) -> Any:
+    """Load a CWL v1.1 input file from a serialized YAML string or a YAML object."""
+    if baseuri is None:
+        baseuri = cwl.file_uri(os.getcwd()) + "/"
+    if loadingOptions is None:
+        loadingOptions = cwl.LoadingOptions()
+
+    result, metadata = _inputfile_load(
+        doc,
+        baseuri,
+        loadingOptions,
+    )
+    return result
+
+
+def load_inputfile_by_string(
+    string: Any,
+    uri: str,
+    loadingOptions: Optional[cwl.LoadingOptions] = None,
+) -> Any:
+    """Load a CWL v1.1 input file from a serialized YAML string."""
+    yaml = yaml_no_ts()
+    result = yaml.load(string)
+    add_lc_filename(result, uri)
+
+    if loadingOptions is None:
+        loadingOptions = cwl.LoadingOptions(fileuri=uri)
+
+    result, metadata = _inputfile_load(
+        result,
+        uri,
+        loadingOptions,
+    )
+    return result
+
+
+def load_inputfile_by_yaml(
+    yaml: Any,
+    uri: str,
+    loadingOptions: Optional[cwl.LoadingOptions] = None,
+) -> Any:
+    """Load a CWL v1.1 input file from a YAML object."""
+    add_lc_filename(yaml, uri)
+
+    if loadingOptions is None:
+        loadingOptions = cwl.LoadingOptions(fileuri=uri)
+
+    result, metadata = _inputfile_load(
+        yaml,
+        uri,
+        loadingOptions,
+    )
+    return result
 
 
 def merge_flatten_type(src: Any) -> Any:
@@ -325,7 +393,7 @@ def merge_flatten_type(src: Any) -> Any:
         return [merge_flatten_type(t) for t in src]
     if isinstance(src, cwl.ArraySchema):
         return src
-    return cwl.ArraySchema(type="array", items=src)
+    return cwl.ArraySchema(type_="array", items=src)
 
 
 def type_for_step_input(
@@ -343,9 +411,9 @@ def type_for_step_input(
                 cast(str, step_input.id).split("#")[-1]
                 == cast(str, in_.id).split("#")[-1]
             ):
-                input_type = step_input.type
+                input_type = step_input.type_
                 if step.scatter is not None and in_.id in aslist(step.scatter):
-                    input_type = cwl.ArraySchema(items=input_type, type="array")
+                    input_type = cwl.ArraySchema(items=input_type, type_="array")
                 return input_type
     return "Any"
 
@@ -363,15 +431,15 @@ def type_for_step_output(
                 output.id.split("#")[-1].split("/")[-1]
                 == sourcename.split("#")[-1].split("/")[-1]
             ):
-                output_type = output.type
+                output_type = output.type_
                 if step.scatter is not None:
                     if step.scatterMethod == "nested_crossproduct":
                         for _ in range(len(aslist(step.scatter))):
                             output_type = cwl.ArraySchema(
-                                items=output_type, type="array"
+                                items=output_type, type_="array"
                             )
                     else:
-                        output_type = cwl.ArraySchema(items=output_type, type="array")
+                        output_type = cwl.ArraySchema(items=output_type, type_="array")
                 return output_type
     raise ValidationException(
         "param {} not found in {}.".format(
@@ -386,57 +454,51 @@ def type_for_source(
     sourcenames: Union[str, List[str]],
     parent: Optional[cwl.Workflow] = None,
     linkMerge: Optional[str] = None,
-    pickValue: Optional[str] = None,
 ) -> Any:
     """Determine the type for the given sourcenames."""
     scatter_context: List[Optional[Tuple[int, str]]] = []
     params = param_for_source_id(process, sourcenames, parent, scatter_context)
     if not isinstance(params, list):
-        new_type = params.type
+        new_type = params.type_
         if scatter_context[0] is not None:
             if scatter_context[0][1] == "nested_crossproduct":
                 for _ in range(scatter_context[0][0]):
-                    new_type = cwl.ArraySchema(items=new_type, type="array")
+                    new_type = cwl.ArraySchema(items=new_type, type_="array")
             else:
-                new_type = cwl.ArraySchema(items=new_type, type="array")
+                new_type = cwl.ArraySchema(items=new_type, type_="array")
         if linkMerge == "merge_nested":
-            new_type = cwl.ArraySchema(items=new_type, type="array")
+            new_type = cwl.ArraySchema(items=new_type, type_="array")
         elif linkMerge == "merge_flattened":
             new_type = merge_flatten_type(new_type)
-        if pickValue is not None:
-            if isinstance(new_type, cwl.ArraySchema):
-                if pickValue in ["first_non_null", "the_only_non_null"]:
-                    new_type = new_type.items
         return new_type
     new_type = []
     for p, sc in zip(params, scatter_context):
         if isinstance(p, str) and not any(_compare_type(t, p) for t in new_type):
             cur_type = p
-        elif hasattr(p, "type") and not any(_compare_type(t, p.type) for t in new_type):
-            cur_type = p.type
+        elif hasattr(p, "type_") and not any(
+            _compare_type(t, p.type_) for t in new_type
+        ):
+            cur_type = p.type_
         else:
             cur_type = None
         if cur_type is not None:
             if sc is not None:
                 if sc[1] == "nested_crossproduct":
                     for _ in range(sc[0]):
-                        cur_type = cwl.ArraySchema(items=cur_type, type="array")
+                        cur_type = cwl.ArraySchema(items=cur_type, type_="array")
                 else:
-                    cur_type = cwl.ArraySchema(items=cur_type, type="array")
+                    cur_type = cwl.ArraySchema(items=cur_type, type_="array")
             new_type.append(cur_type)
     if len(new_type) == 1:
         new_type = new_type[0]
     if linkMerge == "merge_nested":
-        new_type = cwl.ArraySchema(items=new_type, type="array")
+        return cwl.ArraySchema(items=new_type, type_="array")
     elif linkMerge == "merge_flattened":
-        new_type = merge_flatten_type(new_type)
+        return merge_flatten_type(new_type)
     elif isinstance(sourcenames, List) and len(sourcenames) > 1:
-        new_type = cwl.ArraySchema(items=new_type, type="array")
-    if pickValue is not None:
-        if isinstance(new_type, cwl.ArraySchema):
-            if pickValue in ["first_non_null", "the_only_non_null"]:
-                new_type = new_type.items
-    return new_type
+        return cwl.ArraySchema(items=new_type, type_="array")
+    else:
+        return new_type
 
 
 def param_for_source_id(
